/******************************************************************************
*
* (c) Copyright 2011 Xilinx, Inc. All rights reserved.
*
* This file contains confidential and proprietary information of Xilinx, Inc.
* and is protected under U.S. and international copyright and other
* intellectual property laws.
*
* DISCLAIMER
* This disclaimer is not a license and does not grant any rights to the
* materials distributed herewith. Except as otherwise provided in a valid
* license issued to you by Xilinx, and to the maximum extent permitted by
* applicable law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL
* FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS,
* IMPLIED, OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF
* MERCHANTABILITY, NON-INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE;
* and (2) Xilinx shall not be liable (whether in contract or tort, including
* negligence, or under any other theory of liability) for any loss or damage
* of any kind or nature related to, arising under or in connection with these
* materials, including for any direct, or any indirect, special, incidental,
* or consequential loss or damage (including loss of data, profits, goodwill,
* or any type of loss or damage suffered as a result of any action brought by
* a third party) even if such damage or loss was reasonably foreseeable or
* Xilinx had been advised of the possibility of the same.
*
* CRITICAL APPLICATIONS
* Xilinx products are not designed or intended to be fail-safe, or for use in
* any application requiring fail-safe performance, such as life-support or
* safety devices or systems, Class III medical devices, nuclear facilities,
* applications related to the deployment of airbags, or any other applications
* that could lead to death, personal injury, or severe property or
* environmental damage (individually and collectively, "Critical
* Applications"). Customer assumes the sole risk and liability of any use of
* Xilinx products in Critical Applications, subject only to applicable laws
* and regulations governing limitations on product liability.
*
* THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE
* AT ALL TIMES.
*
*******************************************************************************/
/*****************************************************************************/
/**
*
* @file sd.c
*
* Contains code for the SD card FLASH functionality.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date        Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a jz  04/28/11 Initial release
*
* </pre>
*
* @note
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "top_level.h"
#include "xstatus.h"
#include "mio.h"
#include "ff.h"
#include "sd.h"

/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

//#undef  debug_xil_printf

//#define  debug_xil_printf                       xil_printf

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

extern u32 FlashReadBaseAddress;


static FIL fil;                                /* File object */
static FATFS fatfs;
static char buffer[32];
static char *boot_file = buffer;
/******************************************************************************/
/******************************************************************************/
/**
*
* This function initializes the controller for the SD FLASH interface.
*
* @param    filename of the file that is to be used
*
* @return   XST_SUCCESS if the controller initializes correctly
*           XST_FAILURE if the controller fails to initializes correctly
*
* @note    none.
*
****************************************************************************/
u32 InitSD(char *filename)
{
	FRESULT rc;
#if 0
	u32 MioRegAddr;
	u32 PinIndex;

	MioRegAddr = MIO_REG_OFFSET;

	/* Set up the access to the SD Device */
	/* SD start pin */
	MioRegAddr += MIO_SDIO_0_REG_OFFSET;

	for (PinIndex = 0; PinIndex < MIO_SDIO_NUM_PINS; PinIndex++) {

		/* set NOR enable */
		MioOut32((MioRegAddr + (PinIndex * 4)),
		    MIO_ENABLE_SDIO_MASK);
	}
#endif

	/* Register volume work area, initialize device */
	rc = f_mount(0, &fatfs);
	debug_xil_printf("SD: rc= %.8x\n\r", rc);
	if (rc != FR_OK) {
		return XST_FAILURE;
	}

	strcpy_rom(buffer, filename);
	boot_file = filename;
	FlashReadBaseAddress = XPSS_SD_BASEADDR;

	rc = f_open(&fil, boot_file, FA_READ);
	if (rc) {
		debug_xil_printf("SD: Unable to open file %s: %d\n", boot_file, rc);
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/******************************************************************************/
/**
*
* This function provides the SD FLASH interface for the Simplified header
* functionality.
*
* @param    SourceAddress is address in FLASH data space
*           DestinationAddress is address in OCM data space
*           LengthBytes is the number of bytes to move
*
* @return   XST_SUCCESS if the write completes correctly
*           XST_FAILURE if the write fails to completes correctly
*
* @note    none.
*
****************************************************************************/
u32 SDAccess( u32 SourceAddress, u32 DestinationAddress, u32 LengthBytes)
{
	FRESULT rc;                             /* Result code */
	UINT br;
	int  len;

#if 0
	debug_xil_printf("SDAccess: SourceAddress: 0x%08x "
        "DestinationAddress: 0x%08x LengthBytes: 0x%x\n",
	    SourceAddress, DestinationAddress, LengthBytes);
#endif

	rc = f_lseek(&fil, SourceAddress);
	if (rc) {
		debug_xil_printf("SD: Unable to seek to %x\n", SourceAddress);
		return XST_FAILURE;
	}

	//debug_xil_printf("SD: seeked file, to read\n\r");
	len = 0;

	rc = f_read(&fil, (void*)DestinationAddress, LengthBytes, &br);

	if (rc) {
		debug_xil_printf("*** ERROR: f_read returned %d\r\n", rc);
	}

	return XST_SUCCESS;
}

/******************************************************************************/
/**
*
* This function closes the file object
*
****************************************************************************/
void ReleaseSD(void) {
	
	f_close(&fil);
	return;
}
